package se.chalmers.tdv.ddmin;

import java.util.Collection;

public abstract class DDTest<T> {

	/**
	 * Run the test on the input dictated by the circumstances. Should not
	 * change the list or elements in it. I.e., if changes are necessary, the
	 * method should make a deep copy first.
	 * 
	 * @param circumstances
	 *            Defining input to the test.
	 * @return One of {@link DDMin#PASS}, {@link DDMin#FAIL} or
	 *         {@link DDMin#UNRESOLVED}.
	 */
	public abstract int test(Collection<T> circumstances);

	/**
	 * Optional implementation to check whether the list of provided
	 * circumstances constitutes a valid test input.
	 * 
	 * @param inp
	 *            List of circumstances forming the input for the test.
	 * @return True by default.
	 */
	public boolean validInput(Collection<T> circumstances) {
		return true;
	}

}
