package se.chalmers.tdv.queue;

import java.util.Random;

import se.chalmers.tdv.testing.StatefulTestCase;

public class QueueTestSuite {

	static Random rand = new Random();

	public static final QueueTest testRandom = new QueueTest() {

		public String name() {
			return "random";
		}

		public void initialize(
				StatefulTestCase<Queue, QueueAction, Integer> testCase) {
			testCase.setCommands(new QueueAction[] {}, new QueueAction[] {});
		}
	};

	/** front() <==> output(null) (on empty list) */
	public static final QueueTest testFrontEmpty = new QueueTest() {

		public String name() {
			return "frontEmpty";
		}

		public boolean withPrefix() {
			return false;
		}

		public void initialize(
				StatefulTestCase<Queue, QueueAction, Integer> testCase) {
		}
	};

	/** add(A), front <==> out(A) (on empty queue) */
	public static final QueueTest testFront = new QueueTest() {

		public String name() {
			return "front";
		}

		public void initialize(
				StatefulTestCase<Queue, QueueAction, Integer> testCase) {
		}
	};

	/** add(A), add(B), front <==> add(A), front, add(B) */
	public static final QueueTest testFrontAddTwo = new QueueTest() {

		public String name() {
			return "frontTwo";
		}

		public void initialize(
				StatefulTestCase<Queue, QueueAction, Integer> testCase) {
		}
	};

	/** add(A), remove <==> no action (on empty queue) */
	public static final QueueTest testRemove = new QueueTest() {

		public String name() {
			return "remove";
		}

		public void initialize(
				StatefulTestCase<Queue, QueueAction, Integer> testCase) {
		}
	};

	/** add(A), add(B), remove <==> add(A), remove, add(B) */
	public static final QueueTest testRemoveAddTwo = new QueueTest() {

		public String name() {
			return "removeAddTwo";
		}

		public void initialize(
				StatefulTestCase<Queue, QueueAction, Integer> testCase) {
		}
	};

	/** Runs until all test are successful or a bug is found. */
	public static void runAllTests() {
		int NTEST = 1000;

		QueueTest[] tests = { testRandom, testFrontEmpty, testFront,
				testFrontAddTwo, testRemove, testRemoveAddTwo };

		for (QueueTest test : tests) {
			System.out.print("Testing " + test.name() + "... ");
			int i = 1;
			for (; i <= NTEST; i++) {
				QueueTestCase testCase = new QueueTestCase(test);
				if (!testCase.initializeAndRunTest()) {
					System.out.println("\nBug found after " + i + " tests:");

					testCase.minimizeContext();
					testCase.print();
					System.out.println();
					break;
				}
			}
			if (i > NTEST)
				System.out.println("OK " + NTEST + " tests.");
		}

	}

	public static void main(String[] arg) {
		runAllTests();
	}
}
