package se.chalmers.tdv.testing;

/**
 * Generic class describing the methods that should be provided by each test.
 * 
 * @param <T>
 *            The type of the object under test.
 * @param <Action>
 *            A data structure representing the different actions that can be
 *            performed on the type of object under test
 * @param <Out>
 *            Type of potential output values.
 */
public abstract class StatefulTest<T, Action, Out> {

	/** The name of this test. */
	public abstract String name();

	/** Called once for each test case. */
	public abstract void initialize(StatefulTestCase<T, Action, Out> testCase);

	/** Returns true if the provided test case passes this test. */
	public boolean test(StatefulTestCase<T, Action, Out> testCase) {
		return testCase.testObservableEquivalence()
				&& testCase.outputArray1.equals(testCase.outputArray2);
	}

	/** Returns true if the test can be run with a random prefix. */
	public boolean withPrefix() {
		return true;
	}

	/** Returns true if the test can be run with a random suffix. */
	public boolean withSuffix() {
		return true;
	}
}
